import { AsyncComponent } from 'vue';
import type { DefaultComputed } from 'vue/types/options.js';
import type { DefaultData } from 'vue/types/options.js';
import type { DefaultMethods } from 'vue/types/options.js';
import type { Node as Node_2 } from '@nextcloud/files';
import { PropType } from 'vue';

/**
 * FilePicker Vue component (implemented as async component)
 * @example
 * ```vue
 * <template>
 *   <FilePicker name="Select a file" :buttons="buttons" />
 * </template>
 * <script setup lang="ts">
 *   import { FilePickerVue as FilePicker, type IFilePickerButton } from '@nextcloud/dialogs'
 *   const buttons: IFilePickerButton[] = [{
 *     label: 'Pick',
 *     callback: (nodes) => console.log('Picked', nodes)
 *   }]
 * </script>
 */
export declare const FilePickerVue: AsyncComponent<DefaultData<never>, DefaultMethods<never>, DefaultComputed, {
    name: {
        type: PropType<string>;
        required: true;
    };
    buttons: {
        type: PropType<IFilePickerButtonFactory | IFilePickerButton[]>;
        required: true;
    };
    path: {
        type: PropType<string>;
        default: undefined;
    };
    allowPickDirectory: {
        type: PropType<boolean>;
        default: boolean;
    };
    multiselect: {
        type: PropType<boolean>;
        default: boolean;
    };
    disabledNavigation: {
        type: PropType<boolean>;
        default: boolean;
    };
    container: {
        type: PropType<string>;
        default: string;
    };
    filterFn: {
        type: PropType<IFilePickerFilter>;
        default: undefined;
    };
    mimetypeFilter: {
        type: PropType<string[]>;
        default: () => never[];
    };
}>;

/**
 * Interface for defining buttons passed to the Dialog component
 * See NcDialogButton
 */
declare interface IDialogButton {
    /** Label of the button */
    label: string;
    /** Callback on button click */
    callback: () => void;
    /**
     * Optional Icon for the button
     * Should be a SVG image as raw string
     */
    icon?: string;
    /**
     * Button type
     * @see https://nextcloud-vue-components.netlify.app/#/Components/NcButton
     */
    type?: 'primary' | 'secondary' | 'error' | 'warning' | 'success';
}

/**
 * Interface to define buttons of the FilePicker component
 * The buttons are based on the Dialog buttons but the callback gets the array of selected nodes
 */
export declare interface IFilePickerButton extends Omit<IDialogButton, 'callback'> {
    /**
     * Callback on button click
     *
     * @param nodes Array of `@nextcloud/files` Nodes that were selected
     */
    callback: (nodes: Node_2[]) => void;
}

export declare type IFilePickerButtonFactory = (selectedNodes: Node_2[], currentPath: string, currentView: string) => IFilePickerButton[];

/**
 * Type of filter functions to filter the FilePicker's file list
 */
export declare type IFilePickerFilter = (node: Node_2) => boolean;

export { }
